<?php
/**
 * Loulou66
 * LpsBestSellers module for Prestashop
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php*
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Loulou66.fr <contact@loulou66.fr>
 *  @copyright loulou66.fr
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
use PrestaShop\PrestaShop\Adapter\BestSales\BestSalesProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

if (!defined('_PS_VERSION_')) {
    exit;
}

class LpsBestSellers extends Module implements WidgetInterface
{
    public $templateFile;

    public function __construct()
    {
        $this->name = 'lpsbestsellers';
        $this->author = 'Loulou66';
        $this->version = '8.0.1';
        $this->tab = 'front_office_features';
        $this->ps_versions_compliancy = ['min' => '1.7.0', 'max' => _PS_VERSION_];
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('LPS Top-sellers Carousel');
        $this->description = $this->l('Add a block displaying your store top-selling products as a CAROUSEL');
        $this->templateFile = 'module:lpsbestsellers/views/templates/hook/lpsbestsellers.tpl';
        $this->module_key = '569a6fe544116d168a93006027cb0974';
    }

    public function install()
    {
        $this->_clearCache('*');
        return parent::install()
            && Configuration::updateValue('LPSBESTSELLERS_ENABLE', 1)
            && Configuration::updateValue('LPSBESTSELLERS_PROD_DISPLAY', 20)
            && Configuration::updateValue('LPSBESTSELLERS_PROD_PER_ROW', 4)
            && Configuration::updateValue('LPSBESTSELLERS_LOOP', 1)
            && Configuration::updateValue('LPSBESTSELLERS_AUTOPLAY', 1)
            && Configuration::updateValue('LPSBESTSELLERS_RANDOM', 0)
            && $this->registerHook('actionAdminControllerSetMedia')
            && $this->registerHook('actionFrontControllerSetMedia')
            && $this->registerHook('actionOrderStatusPostUpdate')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
            && $this->registerHook('displayHome')
            && $this->registerHook('displayFooterProduct')
            && ProductSale::fillProductSales();
    }

    public function uninstall()
    {
        $this->_clearCache('*');
        return parent::uninstall()
            && Configuration::deleteByName('LPSBESTSELLERS_ENABLE')
            && Configuration::deleteByName('LPSBESTSELLERS_PROD_DISPLAY')
            && Configuration::deleteByName('LPSBESTSELLERS_PROD_PER_ROW')
            && Configuration::deleteByName('LPSBESTSELLERS_LOOP')
            && Configuration::deleteByName('LPSBESTSELLERS_AUTOPLAY')
            && Configuration::deleteByName('LPSBESTSELLERS_RANDOM');
    }

    public function isUsingNewTranslationSystem()
    {
        return false;
    }

    public function hookActionProductAdd($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionProductDelete($params)
    {
        $this->_clearCache('*');
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $this->_clearCache('*');
    }

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        return parent::_clearCache($this->templateFile);
    }

    public function hookActionAdminControllerSetMedia($params)
    {
        if ($this->context->controller->controller_name == 'AdminModules' &&
            Tools::getValue('configure') == $this->name) {
            $this->context->controller->addCSS($this->_path . 'views/css/' . $this->name . 'admin.css');
            $this->context->controller->addJS($this->_path . 'views/js/' . $this->name . 'admin.js');
        }
    }

    public function getContent()
    {
        $html = '';
        if (Tools::isSubmit('submitlpsbestsellers')) {
            if (!Validate::isInt((int) Tools::getValue('LPSBESTSELLERS_PROD_DISPLAY'))
                || (int) Tools::getValue('LPSBESTSELLERS_PROD_DISPLAY') <= 0
            ) {
                $html .= $this->displayError(
                    $this->l('The « Products to display » field is invalid. Please enter a positive number.')
                );
            } else {
                Configuration::updateValue('LPSBESTSELLERS_ENABLE', (int) Tools::getValue('LPSBESTSELLERS_ENABLE'));
                Configuration::updateValue(
                    'LPSBESTSELLERS_PROD_DISPLAY',
                    (int) Tools::getValue('LPSBESTSELLERS_PROD_DISPLAY')
                );
                Configuration::updateValue(
                    'LPSBESTSELLERS_PROD_PER_ROW',
                    (int) Tools::getValue('LPSBESTSELLERS_PROD_PER_ROW')
                );
                Configuration::updateValue('LPSBESTSELLERS_LOOP', (int) Tools::getValue('LPSBESTSELLERS_LOOP'));
                Configuration::updateValue('LPSBESTSELLERS_AUTOPLAY', (int) Tools::getValue('LPSBESTSELLERS_AUTOPLAY'));
                Configuration::updateValue('LPSBESTSELLERS_RANDOM', (int) Tools::getValue('LPSBESTSELLERS_RANDOM'));
                $this->_clearCache('*');
                $html .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $html . $this->renderForm();
    }

    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable Carousel'),
                        'name' => 'LPSBESTSELLERS_ENABLE',
                        'is_bool' => true,
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Products to display'),
                        'name' => 'LPSBESTSELLERS_PROD_DISPLAY',
                        'desc' => $this->l('Determine the total number of products to display in the carousel'),
                        'hint' => $this->l('Determine the total number of products to display in the carousel'),
                        'form_group_class' => 'LPSBESTSELLERS_ENABLE',
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Products visible'),
                        'html_content' => $this->displayProdPerRow(),
                        'name' => '',
                        'desc' => $this->l('Define the number of products visible'),
                        'hint' => $this->l('Define the number of products visible'),
                        'form_group_class' => 'LPSBESTSELLERS_ENABLE',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Loop'),
                        'name' => 'LPSBESTSELLERS_LOOP',
                        'is_bool' => true,
                        'form_group_class' => 'LPSBESTSELLERS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Autoplay'),
                        'name' => 'LPSBESTSELLERS_AUTOPLAY',
                        'is_bool' => true,
                        'form_group_class' => 'LPSBESTSELLERS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Random display'),
                        'name' => 'LPSBESTSELLERS_RANDOM',
                        'hint' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'desc' => $this->l('Enable if you wish the products to be displayed randomly'),
                        'is_bool' => true,
                        'form_group_class' => 'LPSBESTSELLERS_ENABLE',
                        'values' => [
                           [
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG')
            : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitlpsbestsellers';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) .
            '&configure=' . $this->name .
            '&tab_module=' . $this->tab .
            '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        return $helper->generateForm([$fields_form]);
    }

    public function getConfigFieldsValues()
    {
        return [
            'LPSBESTSELLERS_ENABLE' => (int) Tools::getValue(
                'LPSBESTSELLERS_ENABLE',
                Configuration::get('LPSBESTSELLERS_ENABLE'),
            ),
            'LPSBESTSELLERS_PROD_DISPLAY' => (int) Tools::getValue(
                'LPSBESTSELLERS_PROD_DISPLAY',
                Configuration::get('LPSBESTSELLERS_PROD_DISPLAY')
            ),
            'LPSBESTSELLERS_LOOP' => (int) Tools::getValue(
                'LPSBESTSELLERS_LOOP',
                Configuration::get('LPSBESTSELLERS_LOOP'),
            ),
            'LPSBESTSELLERS_AUTOPLAY' => (int) Tools::getValue(
                'LPSBESTSELLERS_AUTOPLAY',
                Configuration::get('LPSBESTSELLERS_AUTOPLAY'),
            ),
            'LPSBESTSELLERS_RANDOM' => (int) Tools::getValue(
                'LPSBESTSELLERS_RANDOM',
                Configuration::get('LPSBESTSELLERS_RANDOM'),
            ),
        ];
    }

    public function displayProdPerRow()
    {
        $tpl = _PS_MODULE_DIR_ . $this->name . '/views/templates/admin/lpsbsprodperrow.tpl';
        $lpsbsprodperrow = $this->context->smarty->createTemplate($tpl);
        $lpsbsprodperrow->assign(
            [
                'lpsbs_prodperrow' => (int) Tools::getValue(
                    'LPSBESTSELLERS_PROD_PER_ROW',
                    Configuration::get('LPSBESTSELLERS_PROD_PER_ROW')
                ),
            ]
        );
        return $lpsbsprodperrow->fetch();
    }

    public function hookActionFrontControllerSetMedia()
    {
        if (!$this->active || !Configuration::get('LPSBESTSELLERS_ENABLE')) {
            return false;
        }
        Media::addJsDef(
            [
                'lpsbs_productsPerRow' => Configuration::get('LPSBESTSELLERS_PROD_PER_ROW'),
                'lpsbs_loop' => Configuration::get('LPSBESTSELLERS_LOOP') ? true : false,
                'lpsbs_autoplay' => Configuration::get('LPSBESTSELLERS_AUTOPLAY') ? true : false,
            ]
        );
        $this->context->controller->registerStylesheet(
            'modules-lpsbestsellersfrontcss',
            'modules/' . $this->name . '/views/css/lpsbestsellersfront.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselcss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.carousel.css'
        );
        $this->context->controller->registerStylesheet(
            'modules-owlcarouselthemecss',
            'modules/' . $this->name . '/libs/owlcarousel/css/owl.theme.default.css'
        );
        $this->context->controller->registerJavascript(
            'modules-owlcarouseljs',
            'modules/' . $this->name . '/libs/owlcarousel/js/owl.carousel.min.js'
        );
        $this->context->controller->registerJavascript(
            'modules-lpsbestsellersfrontjs',
            'modules/' . $this->name . '/views/js/lpsbestsellers.js'
        );
    }

    public function renderWidget($hookName, array $configuration)
    {
        if (!$this->active || !Configuration::get('LPSBESTSELLERS_ENABLE')) {
            return false;
        }
        $lpsbestsellersrandom = Configuration::get('LPSBESTSELLERS_RANDOM');
        if (!$lpsbestsellersrandom) {
            if (!$this->isCached($this->templateFile, $this->getCacheId('lpsbestsellers'))) {
                $variables = $this->getWidgetVariables($hookName, $configuration);
                if (empty($variables)) {
                    return false;
                }
                $this->context->smarty->assign($variables);
            }
            return $this->fetch($this->templateFile, $this->getCacheId('lpsbestsellers'));
        } else {
            $variables = $this->getWidgetVariables($hookName, $configuration);
            if (empty($variables)) {
                return false;
            }
            $this->context->smarty->assign($variables);
            return $this->fetch($this->templateFile);
        }
    }

    public function getWidgetVariables($hookName, array $configuration)
    {
        $lpsbestsellers = $this->getBestSellers();
        if (count($lpsbestsellers) <= Configuration::get('LPSBESTSELLERS_PROD_PER_ROW')) {
            $this->context->smarty->assign(['lpsbs_hidenav' => true]);
        }
        if (!empty($lpsbestsellers)) {
            return ['lpsbestsellers' => $lpsbestsellers];
        }
        return false;
    }
    protected function getBestSellers()
    {
        $nbProducts = Configuration::get('LPSBESTSELLERS_PROD_DISPLAY');
        $searchProvider = new BestSalesProductSearchProvider($this->context->getTranslator());
        $context = new ProductSearchContext($this->context);
        $query = new ProductSearchQuery();
        $query->setResultsPerPage($nbProducts)->setPage(1);
        $query->setSortOrder(new SortOrder('product_sale', 'quantity', 'desc'));
        $result = $searchProvider->runQuery($context, $query);
        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $products_for_template = [];
        $lpsbestsellersres = $result->getProducts();
        if (Configuration::get('LPSBESTSELLERS_RANDOM')) {
            shuffle($lpsbestsellersres);
        }
        foreach ($lpsbestsellersres as $rawProduct) {
            $products_for_template[] = $presenter->present(
                $presentationSettings,
                $assembler->assembleProduct($rawProduct),
                $this->context->language
            );
        }
        return $products_for_template;
    }
}
